@@ -2,6 +2,8 @@
 import logging
 
 from .agg_gen import AggGen
+from .agg_gen import InterpGen
+from .data.user_data import ClimRCatData
 from .data.user_data import ODAPCatData
 from .data.user_data import UserCatData
 from .data.user_data import UserTiffData
@@ -10,7 +12,7 @@ from .zonal_gen import ZonalGen
 
 __author__ = "Richard McDonald"
 __email__ = "rmcd@usgs.gov"
-__version__ = "0.0.32"
+__version__ = "0.0.33"
 
 logging.getLogger(__name__).addHandler(logging.NullHandler())
 
@@ -19,6 +21,8 @@ __all__ = [
     "AggGen",
     "ZonalGen",
     "ODAPCatData",
+    "ClimRCatData",
     "UserCatData",
+    "InterpGen",
     "UserTiffData",
 ]
