@@ -13,6 +13,7 @@ import numpy as np
 import pandas as pd
 import xarray as xr
 from pyproj import CRS
+from shapely.geometry import box
 
 from gdptools.data.agg_gen_data import AggData
 from gdptools.data.odap_cat_data import CatGrids
@@ -43,6 +44,11 @@ class UserData(ABC):
         """Init class."""
         pass
 
+    @abstractmethod
+    def get_source_subset(self, key: str) -> xr.DataArray:
+        """Abstract method for getting subset of source data."""
+        pass
+
     @abstractmethod
     def prep_wght_data(self) -> WeightData:
         """Abstract interface for generating weight data."""
@@ -64,19 +70,232 @@ class UserData(ABC):
         pass
 
 
+class ClimRCatData(UserData):
+    """Instance of UserData using Climate-R catalog data."""
+
+    def __init__(
+        self: "ClimRCatData",
+        *,
+        cat_dict: dict[str, dict[str, Any]],
+        f_feature: Union[str, Path, gpd.GeoDataFrame],
+        id_feature: str,
+        period: List[str],
+    ) -> None:  # sourcery skip: simplify-len-comparison
+        """Initialize ClimRCatData class.
+
+        This class uses wraps the ClimateR-catalogs developed by Mike Johnson
+        and available here:
+
+        'https://mikejohnson51.github.io/climateR-catalogs/catalog.json'
+
+        This can be queried in pandas to return the dictionary associated with a
+        specific source and variable as in the ClimateR-catalog. The cat_dict arguments
+        are composed of a key defined by the variable name and a dictionary of the
+        corresponding ClimateR-catalog json string from the.
+
+        Args:
+            cat_dict (dict[str, dict]): Parameter metadata from climateR-catalog
+                catalog.
+            f_feature (Union[str, Path, gpd.GeoDataFrame]): GeoDataFself.rotate_dsrame
+                or any path-like object that can be read by geopandas.read_file().
+            id_feature (str): Header in GeoDataFrame to use as index for weights.
+            period (List[str]): List containing date strings defining start and end
+                time slice for aggregation.
+        """
+        logger.info("Initializing ODAPCatData")
+        logger.info("  - loading data")
+        self.cat_dict = cat_dict
+        self.f_feature = f_feature
+        self.id_feature = id_feature
+        self.period = pd.to_datetime(period)
+        self._check_input_dict()
+        self.gdf = _read_shp_file(self.f_feature)
+
+        self._check_id_feature()
+        self.keys = self.get_vars()
+        self.cat_params, self.cat_grid = self._create_odapcats(key=self.keys[0])
+
+        logger.info("  - checking latitude bounds")
+        is_intersect, is_degrees, is_0_360 = _check_for_intersection(
+            cat_params=self.cat_params, cat_grid=self.cat_grid, gdf=self.gdf
+        )
+        self.gdf, self.gdf_bounds = _get_shp_file(
+            shp_file=self.gdf, cat_grid=self.cat_grid, is_degrees=is_degrees
+        )
+        self.rotate_ds = bool((not is_intersect) & is_degrees & (is_0_360))
+
+    def get_source_subset(self, key: str) -> xr.DataArray:
+        """get_source_subset Get subset of source data by key.
+
+        _extended_summary_
+
+        Args:
+            key (str): _description_
+
+        Returns:
+            xr.DataArray: _description_
+        """
+        return _get_data_via_catalog(
+            cat_params=self.cat_params,
+            cat_grid=self.cat_grid,
+            bounds=self.gdf_bounds,
+            begin_date=self.period[0],
+            end_date=self.period[1],
+            rotate_lon=self.rotate_ds,
+        )
+
+    def prep_interp_data(self, key: str, poly_id: int) -> AggData:
+        """Prepare data for line-interpolation."""
+        cp, cg = self._create_odapcats(key=key)
+
+        # Select a feature and make sure it remains a geodataframe
+        feature = self.gdf[
+            self.gdf.loc[:, "geometry"] == self.gdf.loc[:, "geometry"][poly_id]
+        ]
+
+        # Reproject to grid crs and get bounding box
+        crs = cg.proj
+        x_cell_size = cg.resX
+        y_cell_size = cg.resY
+        bbox = box(*feature.to_crs(crs).total_bounds)
+        bounds = np.asarray(
+            bbox.buffer(2 * max(x_cell_size, y_cell_size)).bounds  # type: ignore
+        )
+
+        # Clip grid by x, y and time
+        ds_ss = _get_data_via_catalog(
+            cat_params=cp,
+            cat_grid=cg,
+            bounds=bounds,
+            begin_date=str(self.period[0]),
+            end_date=str(self.period[1]),
+            rotate_lon=self.rotate_ds,
+        )
+
+        return AggData(
+            variable=key,
+            cat_param=cp,
+            cat_grid=cg,
+            da=ds_ss,
+            feature=feature,
+            id_feature=self.id_feature,
+            period=self.period,
+        )
+
+    def prep_agg_data(self, key: str, poly_id: Union[int, None] = None) -> AggData:
+        """Prepare ClimRCatData data for aggregation methods.
+
+        Args:
+            key (str): _description_
+            poly_id (int): ID of the row of the geodataframe
+
+        Returns:
+            AggData: _description_
+        """
+        cp, cg = self._create_odapcats(key=key)
+
+        feature = self.gdf
+
+        ds_ss = _get_data_via_catalog(
+            cat_params=cp,
+            cat_grid=cg,
+            bounds=self.gdf_bounds,
+            begin_date=str(self.period[0]),
+            end_date=str(self.period[1]),
+            rotate_lon=self.rotate_ds,
+        )
+
+        return AggData(
+            variable=key,
+            cat_param=cp,
+            cat_grid=cg,
+            da=ds_ss,
+            feature=feature,
+            id_feature=self.id_feature,
+            period=self.period,
+        )
+
+    def prep_wght_data(self) -> WeightData:
+        """Prepare and return WeightData for weight generation."""
+        # For weight generation we just need param and grid dict from one variable
+        # passed to CatParms and CatGrids.
+        cp, cg = self._create_odapcats(key=self.keys[0])
+        self.ds_ss = _get_data_via_catalog(
+            cat_params=cp,
+            cat_grid=cg,
+            bounds=self.gdf_bounds,
+            begin_date=str(self.period[0]),
+            rotate_lon=self.rotate_ds,
+        )
+        tsrt = time.perf_counter()
+        gdf_grid = _get_cells_poly(
+            xr_a=self.ds_ss,
+            x=cg.X_name,
+            y=cg.Y_name,
+            crs_in=cg.proj,
+        )
+        tend = time.perf_counter()
+        print(f"grid cells generated in {tend-tsrt:0.4f} seconds")
+
+        return WeightData(
+            feature=self.gdf, id_feature=self.id_feature, grid_cells=gdf_grid
+        )
+
+    def get_feature_id(self) -> str:
+        """Return id_feature."""
+        return self.id_feature
+
+    def get_vars(self) -> list[str]:
+        """Return list of param_dict keys, proxy for varnames."""
+        return list(self.cat_dict.keys())
+
+    def _check_input_dict(self: "ClimRCatData") -> None:
+        """Check input cat_dict."""
+        if len(self.cat_dict) < 1:
+            raise ValueError("param_dict should have at least 1 key,value pair")
+
+    def _check_id_feature(self: "ClimRCatData") -> None:
+        """Check id_feature in gdf."""
+        if self.id_feature not in self.gdf.columns[:]:
+            raise ValueError(
+                f"shp_poly_idx: {self.id_feature} not in gdf"
+                f" columns: {self.gdf.columns}"
+            )
+
+    def _create_odapcats(self: "ClimRCatData", key: str):
+        key_dict = self.cat_dict.get(key)
+        cp = CatParams(
+            id=key_dict.get("id"),
+            URL=key_dict.get("URL"),
+            varname=key_dict.get("varname"),
+            long_name=key_dict.get("description"),
+            T_name=key_dict.get("T_name"),
+            units=key_dict.get("units"),
+        )
+        cg = CatGrids(
+            X_name=key_dict.get("X_name"),
+            Y_name=key_dict.get("Y_name"),
+            proj=key_dict.get("crs"),
+            resX=key_dict.get("resX"),
+            resY=key_dict.get("resY"),
+            toptobottom=key_dict.get("toptobottom"),
+        )
+        return cp, cg
+
+
 class ODAPCatData(UserData):
     """Instance of UserData using OPeNDAP catalog data."""
 
     def __init__(
         self,
-        *,
+        *,  # all arguments below must be defined as a keyword argument
         param_dict: dict[str, dict[str, Any]],
         grid_dict: dict[str, dict[str, Any]],
         f_feature: Union[str, Path, gpd.GeoDataFrame],
         id_feature: str,
         period: List[str],
     ) -> None:
-        """Initialize PrepODAPCatData class.
+        """Initialize ODAPCatData class.
 
         This class uses a parameter and grid json catalog developed by Mike Johnson
         and available here:
@@ -93,7 +312,7 @@ class ODAPCatData(UserData):
         Args:
             param_dict (dict[str, dict]): Parameter metadata from OPeNDAP catalog.
             grid_dict (dict[str, dict]): Grid metadata from OPeNDAP catalog.
-            f_feature (Union[str, Path, gpd.GeoDataFrame]): GeoDataFrame or any
+            f_feature (Union[str, Path, gpd.GeoDataFrame]): GeoDataFself.rotate_dsrame or any
                 path-like object that can be read by geopandas.read_file().
             id_feature (str): Header in GeoDataFrame to use as index for weights.
             period (List[str]): List containing date strings defining start and end
@@ -107,11 +326,8 @@ class ODAPCatData(UserData):
         self.id_feature = id_feature
         self.period = pd.to_datetime(period)
         self._check_input_dicts()
-        self.gdf = _read_shp_file(self.f_feature)
-
-        self._check_id_feature()
         self.keys = self.get_vars()
-        self._check_input_dicts
+        self.gdf = _read_shp_file(self.f_feature)
         cp = self._create_catparam(key=self.keys[0])
         cg = self._create_catgrid(key=self.keys[0])
         logger.info("  - checking latitude bounds")
@@ -119,10 +335,34 @@ class ODAPCatData(UserData):
             cat_params=cp, cat_grid=cg, gdf=self.gdf
         )
         self.gdf, self.gdf_bounds = _get_shp_file(
-            shp_file=self.gdf, cat_grid=cg, is_degrees=is_degrees
+            shp_file=self.f_feature, cat_grid=cg, is_degrees=is_degrees
         )
+        self._check_id_feature()
         self.rotate_ds = bool((not is_intersect) & is_degrees & (is_0_360))
 
+    def get_source_subset(self, key: str) -> xr.DataArray:
+        """get_source_subset Get data subset from source by key.
+
+        _extended_summary_
+
+        Args:
+            key (str): _description_
+
+        Returns:
+            xr.DataArray: _description_
+        """
+        cat_params = CatParams(**self.param_dict[key])
+        cat_grid = CatGrids(**self.grid_dict[key])
+        # run check on intersection of shape features and gridded data
+        return _get_data_via_catalog(
+            cat_params=cat_params,
+            cat_grid=cat_grid,
+            bounds=self.gdf_bounds,
+            begin_date=self.period[0],
+            end_date=self.period[1],
+            rotate_lon=self.rotate_ds,
+        )
+
     def get_feature_id(self) -> str:
         """Return id_feature."""
         return self.id_feature
@@ -131,11 +371,59 @@ class ODAPCatData(UserData):
         """Return list of param_dict keys, proxy for varnames."""
         return list(self.param_dict.keys())
 
-    def prep_agg_data(self, key: str) -> AggData:
+    def prep_interp_data(self, key: str, poly_id: int) -> AggData:
+        """Prep AggData from UserData.
+
+        Args:
+            key (str): Name of the xarray grided data variable
+            poly_id (int): ID number of the geodataframe geometry to clip the
+                gridded data to
+
+        Returns:
+            AggData: An instance of the AggData class
+        """
+        cp = self._create_catparam(key=key)
+        cg = self._create_catgrid(key=key)
+
+        # Select a feature and make sure it remains a geodataframe
+        feature = self.gdf[
+            self.gdf.loc[:, "geometry"] == self.gdf.loc[:, "geometry"][poly_id]
+        ]
+
+        # Reproject to grid crs and get bounding box
+        crs = cg.proj
+        x_cell_size = cg.resX
+        y_cell_size = cg.resY
+        bbox = box(*feature.to_crs(crs).total_bounds)
+        bounds = np.asarray(
+            bbox.buffer(2 * max(x_cell_size, y_cell_size)).bounds  # type: ignore
+        )
+        # Clip grid by x, y and time
+        ds_ss = _get_data_via_catalog(
+            cat_params=cp,
+            cat_grid=cg,
+            bounds=bounds,
+            begin_date=str(self.period[0]),
+            end_date=str(self.period[1]),
+            rotate_lon=self.rotate_ds,
+        )
+
+        return AggData(
+            variable=key,
+            cat_param=cp,
+            cat_grid=cg,
+            da=ds_ss,
+            feature=feature,
+            id_feature=self.id_feature,
+            period=self.period,
+        )
+
+    def prep_agg_data(self, key: str, poly_id: Union[int, None] = None) -> AggData:
         """Prepare ODAPCatData data for aggregation methods.
 
         Args:
             key (str): _description_
+            poly_id (int): ID of the row of the geodataframe
 
         Returns:
             AggData: _description_
@@ -143,6 +431,8 @@ class ODAPCatData(UserData):
         cp = self._create_catparam(key=key)
         cg = self._create_catgrid(key=key)
 
+        feature = self.gdf
+
         ds_ss = _get_data_via_catalog(
             cat_params=cp,
             cat_grid=cg,
@@ -151,12 +441,13 @@ class ODAPCatData(UserData):
             end_date=str(self.period[1]),
             rotate_lon=self.rotate_ds,
         )
+
         return AggData(
             variable=key,
             cat_param=cp,
             cat_grid=cg,
             da=ds_ss,
-            feature=self.gdf,
+            feature=feature,
             id_feature=self.id_feature,
             period=self.period,
         )
@@ -208,7 +499,7 @@ class ODAPCatData(UserData):
         if len(self.param_dict) != len(self.grid_dict):
             raise ValueError(
                 "Mismatch in key,value pairs. param_dict and grid_dict \
-                should have tehe same number of keys."
+                should have the same number of keys."
             )
         if len(self.param_dict) < 1:
             raise ValueError("param_dict should have at least 1 key,value pair")
@@ -314,6 +605,19 @@ class UserCatData(UserData):
         """Print class name."""
         return f"Class is {cls.__name__}"
 
+    def get_source_subset(self, key: str) -> xr.DataArray:
+        """get_source_subset Get source subset by key.
+
+        _extended_summary_
+
+        Args:
+            key (str): _description_
+
+        Returns:
+            xr.DataArray: _description_
+        """
+        return self.ds[key].sel(**self.agg_subset_dict)
+
     def get_feature_id(self) -> str:
         """Return id_feature."""
         return self.id_feature
@@ -322,6 +626,63 @@ class UserCatData(UserData):
         """Return list of vars in data."""
         return self.var
 
+    def prep_interp_data(self, key: str, poly_id: int) -> AggData:
+        """Prep AggData from UserData.
+
+        Args:
+            key (str): Name of the xarray grided data variable
+            poly_id (int): ID number of the geodataframe geometry to clip the
+                gridded data to
+
+        Returns:
+            AggData: An instance of the AggData class
+        """
+        # Open grid and clip to geodataframe and time window
+        data_ss: xr.DataArray = self.ds[key].sel(**self.agg_subset_dict)  # type: ignore
+        cplist = self._get_user_catparams(key=key, da=data_ss)
+        cglist = self._get_user_catgrids(key=key, da=data_ss)
+        # Create param and grid dictionaries
+        param_dict = dict(zip([key], cplist))  # noqa B905
+        grid_dict = dict(zip([key], cglist))  # noqa B905
+
+        # Select a feature and make sure it remains a geodataframe
+        feature = self.f_feature[
+            self.f_feature.loc[:, "geometry"]
+            == self.f_feature.loc[:, "geometry"][poly_id]
+        ]
+
+        # Reproject the feature to grid crs and get a buffered bounding box
+        crs = grid_dict.get(key)["proj"]
+        x_cell_size = grid_dict.get(key)["resX"]
+        y_cell_size = grid_dict.get(key)["resY"]
+        bbox = box(*feature.to_crs(crs).total_bounds)
+        bounds = np.asarray(
+            bbox.buffer(2 * max(x_cell_size, y_cell_size)).bounds  # type: ignore
+        )
+
+        # Clip grid to time window and line geometry bbox buffer
+        ss_dict = build_subset(
+            bounds=bounds,
+            xname=grid_dict[key]["X_name"],
+            yname=grid_dict[key]["Y_name"],
+            tname=param_dict[key]["T_name"],
+            toptobottom=self.ttb,
+            date_min=str(self.period[0]),
+            date_max=str(self.period[1]),
+        )
+
+        ds_ss = data_ss.sel(**ss_dict)
+
+        return AggData(
+            variable=key,
+            cat_param=CatParams(**param_dict.get(key)),  # type: ignore
+            cat_grid=CatGrids(**grid_dict.get(key)),  # type: ignore
+            da=ds_ss,
+            feature=feature,
+            id_feature=self.id_feature,
+            period=self.period,
+        )
+
     def prep_agg_data(self, key: str) -> AggData:
         """Prep AggData from UserData."""
         logger.info("Agg Data preparation - beginning")
@@ -332,14 +693,14 @@ class UserCatData(UserData):
         param_dict = dict(zip([key], cplist))  # noqa B905
         grid_dict = dict(zip([key], cglist))  # noqa B905
 
-        # print(param_dict)
+        feature = self.f_feature
         logger.info("  - returning AggData")
         return AggData(
             variable=key,
             cat_param=CatParams(**param_dict.get(key)),  # type: ignore
             cat_grid=CatGrids(**grid_dict.get(key)),  # type: ignore
             da=data_ss,
-            feature=self.f_feature,
+            feature=feature,
             id_feature=self.id_feature,
             period=self.period,
         )
@@ -460,6 +821,36 @@ class UserTiffData(UserData):
         self._check_crs()
         self.toptobottom = _get_top_to_bottom(data=self.ds, y_coord=self.y_coord)
 
+    def get_source_subset(self, key: str) -> xr.DataArray:
+        """get_source_subset Get subset of source data.
+
+        _extended_summary_
+
+        Args:
+            key (str): _description_
+
+        Returns:
+            xr.DataArray: _description_
+        """
+        bb_feature = _get_shp_bounds_w_buffer(
+            gdf=self.f_feature,
+            ds=self.ds,
+            crs=self.proj_ds,
+            lon=self.x_coord,
+            lat=self.y_coord,
+        )
+
+        subset_dict = build_subset_tiff(
+            bounds=bb_feature,
+            xname=self.x_coord,
+            yname=self.y_coord,
+            toptobottom=self.toptobottom,
+            bname=self.bname,
+            band=self.band,
+        )
+
+        return self.ds.sel(**subset_dict)  # type: ignore
+
     def get_vars(self) -> list[str]:
         """Return list of varnames."""
         return [self.varname]
