@@ -13,8 +13,10 @@ import numpy as np
 import numpy.typing as npt
 import xarray as xr
 
+from gdptools.data.odap_cat_data import CatClimRItem
 from gdptools.data.odap_cat_data import CatGrids
 from gdptools.data.odap_cat_data import CatParams
+from gdptools.data.odap_cat_data import climr_to_odap
 from gdptools.utils import _check_for_intersection
 from gdptools.utils import _get_data_via_catalog
 from gdptools.utils import _get_shp_file
@@ -165,18 +167,22 @@ def build_subset_tiff_da(
     )
 
 
-def get_data_subset_catalog(
-    cat_params: CatParams,
-    cat_grid: CatGrids,
+def get_data_subset_odap_catalog(
+    param_dict: dict[str, dict[str, Any]],
+    grid_dict: dict[str, dict[str, Any]],
+    key: str,
     shp_file: Union[str, gpd.GeoDataFrame],
     begin_date: str,
     end_date: str,
 ) -> xr.DataArray:
-    """Get xarray subset data.
+    """get_data_subset_odap_catalog Get xarray subset data.
+
+    _extended_summary_
 
     Args:
-        cat_params (CatParams): _description_
-        cat_grid (CatGrids): _description_
+        param_dict (dict[str, dict[str, Any]]): _description_
+        grid_dict (dict[str, dict[str, Any]]): _description_
+        key (str): _description_
         shp_file (Union[str, gpd.GeoDataFrame]): _description_
         begin_date (str): _description_
         end_date (str): _description_
@@ -184,6 +190,8 @@ def get_data_subset_catalog(
     Returns:
         xr.DataArray: _description_
     """
+    cat_params = CatParams(**param_dict[key])
+    cat_grid = CatGrids(**grid_dict[key])
     # run check on intersection of shape features and gridded data
     gdf = _read_shp_file(shp_file)
     is_intersect, is_degrees, is_0_360 = _check_for_intersection(
@@ -204,3 +212,93 @@ def get_data_subset_catalog(
         end_date=end_date,
         rotate_lon=rotate_ds,
     )
+
+
+def get_data_subset_climr_catalog(
+    cat_dict: dict[str, dict[str, Any]],
+    key: str,
+    shp_file: Union[str, gpd.GeoDataFrame],
+    begin_date: str,
+    end_date: str,
+) -> xr.DataArray:
+    """get_data_subset_climr_catalog Get xarray subset data.
+
+    _extended_summary_
+
+    Args:
+        cat_dict (dict[str, dict[str, Any]]): _description_
+        key (str): _description_
+        shp_file (Union[str, gpd.GeoDataFrame]): _description_
+        begin_date (str): _description_
+        end_date (str): _description_
+
+    Returns:
+        xr.DataArray: _description_
+    """
+    cat = CatClimRItem(**cat_dict[key])
+    # run check on intersection of shape features and gridded data
+    cat_params, cat_grid = climr_to_odap(cat)
+    gdf = _read_shp_file(shp_file)
+    is_intersect, is_degrees, is_0_360 = _check_for_intersection(
+        cat_params=cat_params, cat_grid=cat_grid, gdf=gdf
+    )
+
+    # read shapefile, calculate total_bounds, and project to grid's projection
+    gdf, gdf_bounds = _get_shp_file(
+        shp_file=gdf, cat_grid=cat_grid, is_degrees=is_degrees
+    )
+
+    rotate_ds = bool((not is_intersect) & is_degrees & (is_0_360))
+    return _get_data_via_catalog(
+        cat_params=cat_params,
+        cat_grid=cat_grid,
+        bounds=gdf_bounds,
+        begin_date=begin_date,
+        end_date=end_date,
+        rotate_lon=rotate_ds,
+    )
+
+
+def get_data_subset_user_catalog(
+    cat_dict: dict[str, dict[str, Any]],
+    key: str,
+    shp_file: Union[str, gpd.GeoDataFrame],
+    begin_date: str,
+    end_date: str,
+) -> xr.DataArray:
+    """get_data_subset_climr_catalog Get xarray subset data.
+
+    _extended_summary_
+
+    Args:
+        cat_dict (dict[str, dict[str, Any]]): _description_
+        key (str): _description_
+        shp_file (Union[str, gpd.GeoDataFrame]): _description_
+        begin_date (str): _description_
+        end_date (str): _description_
+
+    Returns:
+        xr.DataArray: _description_
+    """
+    cat = CatClimRItem(**cat_dict[key])
+    # run check on intersection of shape features and gridded data
+    cat_params, cat_grid = climr_to_odap(cat)
+    gdf = _read_shp_file(shp_file)
+    is_intersect, is_degrees, is_0_360 = _check_for_intersection(
+        cat_params=cat_params, cat_grid=cat_grid, gdf=gdf
+    )
+
+    # read shapefile, calculate total_bounds, and project to grid's projection
+    gdf, gdf_bounds = _get_shp_file(
+        shp_file=gdf, cat_grid=cat_grid, is_degrees=is_degrees
+    )
+
+    rotate_ds = bool((not is_intersect) & is_degrees & (is_0_360))
+    return _get_data_via_catalog(
+        cat_params=cat_params,
+        cat_grid=cat_grid,
+        bounds=gdf_bounds,
+        begin_date=begin_date,
+        end_date=end_date,
+        rotate_lon=rotate_ds,
+    )
