@@ -3,6 +3,7 @@ from __future__ import annotations
 
 import json
 import logging
+import math
 import re
 import sys
 import time
@@ -12,6 +13,7 @@ from datetime import datetime
 from pathlib import Path
 from typing import Any
 from typing import Dict
+from typing import List
 from typing import Optional
 from typing import Tuple
 from typing import Union
@@ -24,14 +26,11 @@ import numpy.typing as npt
 import pandas as pd
 import pygeos
 import rioxarray as rxr
-import shapely
 import xarray as xr
 from pyproj import CRS
 from shapely.geometry import box
-from shapely.geometry import LineString
-from shapely.geometry import MultiPolygon
+from shapely.geometry import Point
 from shapely.geometry import Polygon
-from shapely.ops import split
 
 from gdptools.data.odap_cat_data import CatGrids
 from gdptools.data.odap_cat_data import CatParams
@@ -50,21 +49,20 @@ def _get_grid_cell_sindex(grid_cells: gpd.GeoDataFrame) -> geopandas.sindex:
 
 
 def _reproject_for_weight_calc(
-    poly: gpd.GeoDataFrame,
-    grid_cells: gpd.GeoDataFrame,
-    grid_out_crs: Any,
+    target_poly: gpd.GeoDataFrame,
+    source_poly: gpd.GeoDataFrame,
     wght_gen_crs: Any,
 ) -> Tuple[gpd.GeoDataFrame, gpd.GeoDataFrame]:
     start = time.perf_counter()
-    # grid_cells.set_crs(grid_in_crs, inplace=True)
-    grid_cells = grid_cells.to_crs(grid_out_crs)
-    poly = poly.to_crs(grid_out_crs)
+    # source_poly.set_crs(grid_in_crs, inplace=True)
+    source_poly = source_poly.to_crs(wght_gen_crs)
+    target_poly = target_poly.to_crs(wght_gen_crs)
     end = time.perf_counter()
     print(
         f"Reprojecting to epsg:{wght_gen_crs} finished in {round(end-start, 2)}"
         " second(s)"
     )
-    return poly, grid_cells
+    return target_poly, source_poly
 
 
 def _check_grid_cell_crs(grid_cells: gpd.GeoDataFrame) -> None:
@@ -79,14 +77,25 @@ def _check_feature_crs(poly: gpd.GeoDataFrame) -> None:
         raise ValueError(error_string)
 
 
-def _check_poly_idx(poly: gpd.GeoDataFrame, poly_idx: str) -> None:
+def _check_target_poly_idx(poly: gpd.GeoDataFrame, poly_idx: str) -> None:
     if poly_idx not in poly.columns:
         error_string = (
-            f"Error: poly_idx ({poly_idx}) is not found in the poly ({poly.columns})"
+            f"Error: target_poly_idx ({poly_idx}) is not found in the poly"
+            f" ({poly.columns})"
         )
         raise ValueError(error_string)
 
 
+def _check_source_poly_idx(poly: gpd.GeoDataFrame, poly_idx: List[str]) -> None:
+    for id in poly_idx:
+        if id not in poly.columns:
+            error_string = (
+                f"Error: source_poly_idx ({id}) is not found in the poly"
+                f" ({poly.columns})"
+            )
+            raise ValueError(error_string)
+
+
 def _get_print_on(numrows: int) -> int:
     """Return an interval to print progress of run_weights() function.
 
@@ -605,7 +614,7 @@ def _get_data_via_catalog(
     Args:
         cat_params (CatParams): _description_
         cat_grid (CatGrids): _description_
-        bounds (np.ndarray): _description_
+        bounds (Tuple[np.double, np.double, np.double, np.double]): _description_
         begin_date (str): _description_
         end_date (Optional[str], optional): _description_. Defaults to None.
         rotate_lon (Optional[bool], optional): _description_. Defaults to False.
@@ -632,7 +641,7 @@ def _get_data_via_catalog(
     return ds[varname].sel(**ss_dict)
 
 
-def _get_wieght_df(wght_file: Union[str, pd.DataFrame], poly_idx: str) -> pd.DataFrame:
+def _get_weight_df(wght_file: Union[str, pd.DataFrame], poly_idx: str) -> pd.DataFrame:
     if isinstance(wght_file, pd.DataFrame):
         # wghts = wght_file.copy()
         wghts = wght_file.astype({"i": int, "j": int, "wght": float, poly_idx: str})
@@ -676,56 +685,6 @@ def _get_dataframe(object: Union[str, pd.DataFrame]) -> pd.DataFrame:
         return object
 
 
-# this function is copied from here:
-# https://github.com/gboeing/osmnx/blob/main/osmnx/utils_geo.py
-# if it turns out to be helpful in speedin up intersections then I'll import it via
-# the package - for now copied in here for development testing.
-def _quadrat_cut_geometry(
-    geometry: Union[shapely.geometry.Polygon, shapely.geometry.MultiPolygon],
-    quadrat_width: float,
-    min_num: Optional[int] = 3,
-) -> shapely.geometry.MultiPolygon:
-    """Split a Polygon or MultiPolygon up into sub-polygons of a specified size.
-
-    Args:
-        geometry (Union[shapely.geometry.Polygon, shapely.geometry.MultiPolygon]): _description_
-        quadrat_width (float): _description_
-        min_num (Optional[int], optional): _description_. Defaults to 3.
-
-    Returns:
-        shapely.geometry.MultiPolygon: _description_
-    """
-    # create n evenly spaced points between the min and max x and y bounds
-    west, south, east, north = geometry.bounds
-    x_num = int(np.ceil((east - west) / quadrat_width) + 1)
-    y_num = int(np.ceil((north - south) / quadrat_width) + 1)
-    x_points = np.linspace(west, east, num=np.max(x_num, min_num))
-    y_points = np.linspace(south, north, num=np.max(y_num, min_num))
-
-    # create a quadrat grid of lines at each of the evenly spaced points
-    vertical_lines = [
-        LineString([(x, y_points[0]), (x, y_points[-1])]) for x in x_points
-    ]
-    horizont_lines = [
-        LineString([(x_points[0], y), (x_points[-1], y)]) for y in y_points
-    ]
-    lines = vertical_lines + horizont_lines
-
-    # recursively split the geometry by each quadrat line
-    geometries = [geometry]
-
-    for line in lines:
-        # split polygon by line if they intersect, otherwise just keep it
-        split_geoms = [
-            split(g, line).geoms if g.intersects(line) else [g] for g in geometries
-        ]
-        # now flatten the list and process these split geoms on the next line in
-        # the list of lines
-        geometries = [g for g_list in split_geoms for g in g_list]
-
-    return MultiPolygon(geometries)
-
-
 def _get_default_val(native_dtype: np.dtype):  # type: ignore
     """Get default fill value based on gridded data dtype.
 
@@ -803,7 +762,7 @@ def _get_top_to_bottom(data: Union[xr.Dataset, xr.DataArray], y_coord: str) -> b
         bool: _description_
     """
     yy = data.coords[y_coord].values
-    return bool(yy[0] <= yy[-1])
+    return yy[0] <= yy[-1]
 
 
 def _get_xr_dataset(ds: Union[str, xr.Dataset]) -> xr.Dataset:
@@ -830,3 +789,153 @@ def _get_geodataframe(f_feature: gpd.GeoDataFrame) -> gpd.GeoDataFrame:
         return gpd.read_file(f_feature)
     else:
         return f_feature
+
+
+def _interpolate_sample_points(
+    geom: gpd.GeoSeries, spacing: Union[float, int], calc_crs: Any, crs: Any
+) -> Tuple[list[float], list[float], list[float]]:
+    """Interpolated points at equal distances along a line.
+
+    Return the interpolated points and their distances from the initial point.
+
+    Args:
+        geom (GeoSeries): Line geometry to pull sample points from
+        spacing (float): The distance in meters between the sample points
+        calc_crs(Any): Coordinate system to calculate interpolated points and distance.
+        crs (Any): Coordinate system to return the points in. EPSG code or Proj 4
+            string.
+
+    Returns:
+        x (Numpy Array): Array of x coordinates of the sample points
+        y (Numpy Array): Array of y coordinates of the sample points
+        dist (Numpy Array): Array of distances from the first point to each of the
+            sample points
+    """
+    # equidistant_crs = "+proj=eqdc +lat_0=39 +lon_0=-96 +lat_1=33 \
+    #     +lat_2=45 +x_0=0 +y_0=0 +datum=NAD83 +units=m +no_defs +type=crs"
+    # rp_geom: gpd.GeoSeries = geom.to_crs(equidistant_crs)
+    rp_geom: gpd.GeoSeries = geom.to_crs(calc_crs)
+    # Get line length
+    length = rp_geom.length.values[0]
+    # Calculate the number of sample points
+    num_points = int(length / spacing) + 1
+    # Create empty numpy arrays for x,y coords
+    x = np.zeros(num_points, dtype=np.double)
+    y = np.zeros(num_points, dtype=np.double)
+    dist = np.zeros(num_points, dtype=np.double)
+    # Find sample points on line  # from nldi_xstools.PathGen
+    d = 0.0
+    index = 0
+    while d < length:
+        point = rp_geom.interpolate(d)
+        # Project to grid crs
+        point = point.to_crs(crs)
+        x[index] = point.x
+        y[index] = point.y
+        dist[index] = d
+        d += spacing
+        index += 1
+
+    return x, y, dist
+
+
+def _get_line_vertices(
+    geom: gpd.GeoDataFrame, crs: Any
+) -> Tuple[list[float], list[float], list[float]]:
+    """Return the vertices and the distance inbetween of a line in a GeoDataFrame.
+
+    Args:
+        geom (GeoDataFrame): A GeoDataFrame with a single line geometry
+        crs (Any): Coordinate system to return the points in. EPSG code or Proj 4 string.
+
+    Returns:
+        Tuple[list[float], list[float], list[float]]: Three list containing the x coords,
+            y coords and distance in meters from the first vertex to each vertex
+    """
+    # project to equidistant crs to calculate distance between vertices
+    equidistant_crs = "+proj=eqdc +lat_0=39 +lon_0=-96 +lat_1=33 +lat_2=45 +x_0=0 \
+        +y_0=0 +datum=NAD83 +units=m +no_defs +type=crs"
+    rp_geom = geom.to_crs(equidistant_crs)
+    x, y = rp_geom.reset_index().geometry[0].geoms[0].coords.xy
+    x = list(x)
+    y = list(y)
+
+    # cal distance between the first vertex and each vertex
+    for i in range(len(x)):
+        if i == 0:
+            dist = [0.0]
+        else:
+            d = dist[i - 1] + math.dist([x[i - 1], y[i - 1]], [x[i], y[i]])
+            dist.append(d)
+
+    # Project line to grid crs and export vertex coords
+    rp_geom = geom.to_crs(crs)
+    x, y = rp_geom.reset_index().geometry[0].geoms[0].coords.xy
+    x = list(x)
+    y = list(y)
+
+    return x, y, dist
+
+
+def _cal_point_stats(data_array: xr.DataArray, stat: str) -> dict[Any]:
+    """Calculate the specified stats from a geodataframe of points.
+
+    Args:
+        data_array: (xr.DataArray): Xarray DataArray of values pulled from
+            a gridded dataset at interpolated points
+        stat (str): A string indicating which statistics to calculated.
+            Options: all, mean, median, std, min, max
+
+    Returns:
+        dict: A dictionary of statistical values
+    """
+    out_vals: dict[Any] = {}
+
+    # Calculate the stats
+    if stat in {"mean", "all"}:
+        out_vals["mean"] = data_array.mean(dim=["pt"]).values
+
+    if stat in {"median", "all"}:
+        out_vals["median"] = data_array.median(dim=["pt"]).values
+
+    if stat in {"std", "all"}:
+        out_vals["std"] = data_array.std(dim=["pt"]).values
+
+    if stat in {"min", "all"}:
+        out_vals["min"] = data_array.min(dim=["pt"]).values
+
+    if stat in {"max", "all"}:
+        out_vals["max"] = data_array.max(dim=["pt"]).values
+
+    return out_vals
+
+
+def _buffer_line(
+    geometry: gpd.GeoSeries,
+    buffer: Union[float, int],
+    proj_feature: CRS,
+    calc_crs: Any,
+) -> gpd.GeoSeries:
+    """Buffer a line segment.
+
+    The line gets reprojected to an AEA projection, so that the buffer can be
+    submitted in meters. Then the buffered geometry it reprojected back to
+    a user specified crs.
+
+    Args:
+        geometry (GeoSeries): Geometry of the query line
+        buffer (float): Value in meters of the diameter of the buffer
+        proj_feature (CRS): Coordinate system of the returned buffer geometry
+        calc_crs: Coordinate system in which to perform the statistical calculations
+
+    Returns:
+        new_geometry (GeoSeries): Geometry of the buffer
+    """
+    return geometry.to_crs(calc_crs).buffer(buffer).to_crs(proj_feature)
+
+
+def _dataframe_to_geodataframe(df: pd.DataFrame, crs: str) -> gpd.GeoDataFrame:
+    """Convert pandas Dataframe to Geodataframe."""
+    geometry: list[Point] = [Point(xy) for xy in zip(df.x, df.y)]  # noqa: B905
+    df = df.drop(["x", "y", "crs"], axis=1)
+    return gpd.GeoDataFrame(df, geometry=geometry, crs=crs)
