@@ -2,6 +2,7 @@
 import time
 from typing import Any
 from typing import Literal
+from typing import Optional
 from typing import Union
 
 import geopandas as gpd
@@ -18,7 +19,7 @@ WEIGHT_GEN_METHODS = Literal["serial", "parallel", "dask"]
 
 seerial: Iterates through polygons to calculate weights.  Sufficient for most cases.
 parallel: Chunks polygons and distributes to available processors.  Provides a
-    substantial speedup when there is a large number of polygons.
+substantial speedup when there is a large number of polygons.
 
 Raises:
     TypeError: If value is not one of "serial" or "parallel".
@@ -33,29 +34,35 @@ class WeightGen:
 
     def __init__(
         self,
+        *,
         user_data: UserData,
         method: WEIGHT_GEN_METHODS,
-        output_file: str,
         weight_gen_crs: Any,
-        jobs: int = -1,
-        verbose: bool = False,
+        output_file: Optional[Union[str, None]] = None,
+        jobs: Optional[int] = -1,
+        verbose: Optional[bool] = False,
     ) -> None:
-        """Weight generation class.
+        """__init__ Weight generation class.
+
+        _extended_summary_
 
         Args:
             user_data (UserData): _description_
-            method (WEIGHT_GEN_METHODS): :data: `gdptools.weights.WeightGen.WEIGHT_GEN_METHODS`
-            output_file (str): _description_
+            method (WEIGHT_GEN_METHODS): _description_
             weight_gen_crs (Any): _description_
-            jobs (int): _description_. Defaults to 1
-            verbose (bool): _description_. Defaults to False.
+            output_file (Optional[Union[str, None]], optional): _description_. Defaults to None.
+            jobs (Optional[int], optional): _description_. Defaults to -1.
+            verbose (Optional[bool], optional): _description_. Defaults to False.
 
         Raises:
             TypeError: _description_
         """
         self._user_data = user_data
         self._method = method
-        self._output_file = output_file
+        if output_file is None:
+            self._output_file = ""
+        else:
+            self._output_file = output_file
         self._weight_gen_crs = weight_gen_crs
         self._jobs = jobs
         self._verbose = verbose
@@ -90,9 +97,11 @@ class WeightGen:
         if intersections:
             print("Saving interesections in weight generation.")
             weights, self._intersections = self.__calc_method.calc_weights(
-                poly=self._weight_data.feature,
-                poly_idx=self._weight_data.id_feature,
-                grid_cells=self._weight_data.grid_cells,
+                target_poly=self._weight_data.feature,
+                target_poly_idx=self._weight_data.id_feature,
+                source_poly=self._weight_data.grid_cells,
+                source_poly_idx=["i_index", "j_index"],
+                source_type="grid",
                 wght_gen_crs=self._weight_gen_crs,
                 filename=self._output_file,
                 intersections=intersections,
@@ -101,9 +110,11 @@ class WeightGen:
             )
         else:
             weights = self.__calc_method.calc_weights(
-                poly=self._weight_data.feature,
-                poly_idx=self._weight_data.id_feature,
-                grid_cells=self._weight_data.grid_cells,
+                target_poly=self._weight_data.feature,
+                target_poly_idx=self._weight_data.id_feature,
+                source_poly=self._weight_data.grid_cells,
+                source_poly_idx=["i_index", "j_index"],
+                source_type="grid",
                 wght_gen_crs=self._weight_gen_crs,
                 filename=self._output_file,
                 intersections=intersections,
