@@ -10,8 +10,10 @@ import numpy.typing as npt
 import pandas as pd
 import pytest
 import xarray as xr
+from pandas.testing import assert_frame_equal
 
 from gdptools import AggGen
+from gdptools import InterpGen
 from gdptools import ODAPCatData
 from gdptools import UserCatData
 from gdptools import WeightGen
@@ -20,8 +22,8 @@ gm_vars = ["aet", "pet", "PDSI"]
 
 
 @pytest.fixture
-def aet_ma_average() -> npt.NDArray[np.double]:
-    """Test values for masked-average calculation."""
+def aet_ma_mean() -> npt.NDArray[np.double]:
+    """Test values for masked-mean calculation."""
     return np.asarray(
         [
             [3.1605408],
@@ -41,8 +43,8 @@ def aet_ma_average() -> npt.NDArray[np.double]:
 
 
 @pytest.fixture
-def aet_average() -> npt.NDArray[np.double]:
-    """Test values for average calculation."""
+def aet_mean() -> npt.NDArray[np.double]:
+    """Test values for mean calculation."""
     return np.asarray(
         [
             [3.1551743],
@@ -61,6 +63,27 @@ def aet_average() -> npt.NDArray[np.double]:
     )
 
 
+@pytest.fixture
+def aet_nan_mean() -> npt.NDArray[np.double]:
+    """Test values for mean calculation."""
+    return np.asarray(
+        [
+            [np.nan],
+            [np.nan],
+            [np.nan],
+            [np.nan],
+            [np.nan],
+            [np.nan],
+            [np.nan],
+            [np.nan],
+            [np.nan],
+            [np.nan],
+            [np.nan],
+            [np.nan],
+        ]
+    )
+
+
 @pytest.fixture
 def param_dict(vars: list[str] = gm_vars) -> dict[str, Any]:
     """Return parameter json."""
@@ -115,8 +138,8 @@ def test_odapcatdata_dask_weights_w_intersections(
     gdf: gpd.GeoDataFrame,
     poly_idx: str,
     wght_gen_proj: Any,
-    aet_ma_average: npt.NDArray[np.double],
-    aet_average: npt.NDArray[np.double],
+    aet_ma_mean: npt.NDArray[np.double],
+    aet_nan_mean: npt.NDArray[np.double],
 ) -> None:
     """Test ODAPCatData."""
     from dask.distributed import Client
@@ -146,7 +169,82 @@ def test_odapcatdata_dask_weights_w_intersections(
     tmpdir = TemporaryDirectory()
     gen_agg = AggGen(
         user_data=user_data,
-        stat_method="masked_average",
+        stat_method="masked_mean",
+        agg_engine="parallel",
+        agg_writer="netcdf",
+        weights=tmpfile.name,
+        out_path=tmpdir.name,
+        file_prefix="test_agg_gen",
+        jobs=4,
+    )
+
+    _ngdf, nvals = gen_agg.calculate_agg()
+    file = Path(tmpdir.name) / "test_agg_gen.nc"
+    print(nvals["aet"].values)
+    assert file.exists()
+
+    np.testing.assert_allclose(
+        aet_ma_mean, nvals["aet"].values, rtol=1e-4, verbose=True  # type: ignore
+    )
+
+    tmpdir2 = TemporaryDirectory()
+    gen_agg = AggGen(
+        user_data=user_data,
+        stat_method="mean",
+        agg_engine="serial",
+        agg_writer="csv",
+        weights=tmpfile.name,
+        out_path=tmpdir2.name,
+        file_prefix="test_agg_gen",
+        jobs=4,
+    )
+
+    _ngdf, nvals2 = gen_agg.calculate_agg()
+    file = Path(tmpdir2.name) / "test_agg_gen.csv"
+    assert file.exists()
+    np.testing.assert_allclose(
+        aet_nan_mean, nvals2["aet"].values, rtol=1e-4, verbose=True
+    )
+
+
+def test_odapcatdata_d(
+    param_dict: dict[str, Any],
+    grid_dict: dict[str, Any],
+    gdf: gpd.GeoDataFrame,
+    poly_idx: str,
+    wght_gen_proj: Any,
+    aet_ma_mean: npt.NDArray[np.double],
+    aet_nan_mean: npt.NDArray[np.double],
+) -> None:
+    """Test ODAPCatData."""
+    from dask.distributed import Client
+
+    client = Client()  # type: ignore
+    user_data = ODAPCatData(
+        param_dict=param_dict,
+        grid_dict=grid_dict,
+        f_feature=gdf,
+        id_feature=poly_idx,
+        period=["1980-01-01", "1980-12-31"],
+    )
+    tmpfile = NamedTemporaryFile()
+
+    gen_weights = WeightGen(
+        user_data=user_data,
+        method="dask",
+        output_file=tmpfile.name,
+        weight_gen_crs=wght_gen_proj,
+        jobs=4,
+    )
+
+    _wghts = gen_weights.calculate_weights()  # noqa: F841
+
+    client.close()  # type: ignore
+
+    tmpdir = TemporaryDirectory()
+    gen_agg = AggGen(
+        user_data=user_data,
+        stat_method="masked_mean",
         agg_engine="parallel",
         agg_writer="netcdf",
         weights=tmpfile.name,
@@ -160,13 +258,13 @@ def test_odapcatdata_dask_weights_w_intersections(
     assert file.exists()
 
     np.testing.assert_allclose(
-        aet_ma_average, nvals[0], rtol=1e-4, verbose=True  # type: ignore
+        aet_ma_mean, nvals["aet"].values, rtol=1e-4, verbose=True
     )
 
     tmpdir2 = TemporaryDirectory()
     gen_agg = AggGen(
         user_data=user_data,
-        stat_method="average",
+        stat_method="mean",
         agg_engine="serial",
         agg_writer="csv",
         weights=tmpfile.name,
@@ -175,97 +273,28 @@ def test_odapcatdata_dask_weights_w_intersections(
         jobs=4,
     )
 
+    # test_arr = np.asarray(
+    #     [
+    #         [3.1551743],
+    #         [0.0121071],
+    #         [41.9124973],
+    #         [68.7980207],
+    #         [81.5847434],
+    #         [92.4489252],
+    #         [48.8779355],
+    #         [37.2405776],
+    #         [27.8365651],
+    #         [64.5483926],
+    #         [52.8072282],
+    #         [1.63630845],
+    #     ]
+    # )
     _ngdf, nvals2 = gen_agg.calculate_agg()
     file = Path(tmpdir2.name) / "test_agg_gen.csv"
     assert file.exists()
-    print(nvals2[0])
-    np.testing.assert_allclose(aet_average, nvals2[0], rtol=1e-4, verbose=True)
-
-
-def test_odapcatdata_d(
-    param_dict: dict[str, Any],
-    grid_dict: dict[str, Any],
-    gdf: gpd.GeoDataFrame,
-    poly_idx: str,
-    wght_gen_proj: Any,
-    aet_ma_average: npt.NDArray[np.double],
-) -> None:
-    """Test ODAPCatData."""
-    from dask.distributed import Client
-
-    client = Client()  # type: ignore
-    user_data = ODAPCatData(
-        param_dict=param_dict,
-        grid_dict=grid_dict,
-        f_feature=gdf,
-        id_feature=poly_idx,
-        period=["1980-01-01", "1980-12-31"],
+    np.testing.assert_allclose(
+        aet_nan_mean, nvals2["aet"].values, rtol=1e-4, verbose=True
     )
-    tmpfile = NamedTemporaryFile()
-
-    gen_weights = WeightGen(
-        user_data=user_data,
-        method="dask",
-        output_file=tmpfile.name,
-        weight_gen_crs=wght_gen_proj,
-        jobs=4,
-    )
-
-    _wghts = gen_weights.calculate_weights()  # noqa: F841
-
-    client.close()  # type: ignore
-
-    tmpdir = TemporaryDirectory()
-    gen_agg = AggGen(
-        user_data=user_data,
-        stat_method="masked_average",
-        agg_engine="parallel",
-        agg_writer="netcdf",
-        weights=tmpfile.name,
-        out_path=tmpdir.name,
-        file_prefix="test_agg_gen",
-        jobs=4,
-    )
-
-    _ngdf, nvals = gen_agg.calculate_agg()
-    file = Path(tmpdir.name) / "test_agg_gen.nc"
-    assert file.exists()
-
-    np.testing.assert_allclose(aet_ma_average, nvals[0], rtol=1e-4, verbose=True)
-
-    tmpdir2 = TemporaryDirectory()
-    gen_agg = AggGen(
-        user_data=user_data,
-        stat_method="average",
-        agg_engine="serial",
-        agg_writer="csv",
-        weights=tmpfile.name,
-        out_path=tmpdir2.name,
-        file_prefix="test_agg_gen",
-        jobs=4,
-    )
-
-    test_arr = np.asarray(
-        [
-            [3.1551743],
-            [0.0121071],
-            [41.9124973],
-            [68.7980207],
-            [81.5847434],
-            [92.4489252],
-            [48.8779355],
-            [37.2405776],
-            [27.8365651],
-            [64.5483926],
-            [52.8072282],
-            [1.63630845],
-        ]
-    )
-    _ngdf, nvals2 = gen_agg.calculate_agg()
-    file = Path(tmpdir2.name) / "test_agg_gen.csv"
-    assert file.exists()
-    print(nvals2[0])
-    np.testing.assert_allclose(test_arr, nvals2[0], rtol=1e-4, verbose=True)
 
 
 def test_odapcatdata_parallel_w_intersections(
@@ -274,8 +303,8 @@ def test_odapcatdata_parallel_w_intersections(
     gdf: gpd.GeoDataFrame,
     poly_idx: str,
     wght_gen_proj: Any,
-    aet_ma_average: npt.NDArray[np.double],
-    aet_average: npt.NDArray[np.double],
+    aet_ma_mean: npt.NDArray[np.double],
+    aet_nan_mean: npt.NDArray[np.double],
 ) -> None:
     """Test ODAPCatData."""
     user_data = ODAPCatData(
@@ -300,7 +329,7 @@ def test_odapcatdata_parallel_w_intersections(
     tmpdir = TemporaryDirectory()
     gen_agg = AggGen(
         user_data=user_data,
-        stat_method="masked_average",
+        stat_method="masked_mean",
         agg_engine="parallel",
         agg_writer="netcdf",
         weights=tmpfile.name,
@@ -313,12 +342,14 @@ def test_odapcatdata_parallel_w_intersections(
     file = Path(tmpdir.name) / "test_agg_gen.nc"
     assert file.exists()
 
-    np.testing.assert_allclose(aet_ma_average, nvals[0], rtol=1e-4, verbose=True)
+    np.testing.assert_allclose(
+        aet_ma_mean, nvals["aet"].values, rtol=1e-4, verbose=True
+    )
 
     tmpdir2 = TemporaryDirectory()
     gen_agg = AggGen(
         user_data=user_data,
-        stat_method="average",
+        stat_method="mean",
         agg_engine="parallel",
         agg_writer="csv",
         weights=tmpfile.name,
@@ -330,8 +361,9 @@ def test_odapcatdata_parallel_w_intersections(
     _ngdf, nvals2 = gen_agg.calculate_agg()
     file = Path(tmpdir2.name) / "test_agg_gen.csv"
     assert file.exists()
-    print(nvals2[0])
-    np.testing.assert_allclose(aet_average, nvals2[0], rtol=1e-4, verbose=True)
+    np.testing.assert_allclose(
+        aet_nan_mean, nvals2["aet"].values, rtol=1e-4, verbose=True
+    )
 
 
 def test_odapcatdata_p(
@@ -340,8 +372,8 @@ def test_odapcatdata_p(
     gdf: gpd.GeoDataFrame,
     poly_idx: str,
     wght_gen_proj: Any,
-    aet_ma_average: npt.NDArray[np.double],
-    aet_average: npt.NDArray[np.double],
+    aet_ma_mean: npt.NDArray[np.double],
+    aet_nan_mean: npt.NDArray[np.double],
 ) -> None:
     """Test ODAPCatData."""
     user_data = ODAPCatData(
@@ -366,7 +398,7 @@ def test_odapcatdata_p(
     tmpdir = TemporaryDirectory()
     gen_agg = AggGen(
         user_data=user_data,
-        stat_method="masked_average",
+        stat_method="masked_mean",
         agg_engine="parallel",
         agg_writer="netcdf",
         weights=tmpfile.name,
@@ -379,12 +411,14 @@ def test_odapcatdata_p(
     file = Path(tmpdir.name) / "test_agg_gen.nc"
     assert file.exists()
 
-    np.testing.assert_allclose(aet_ma_average, nvals[0], rtol=1e-4, verbose=True)
+    np.testing.assert_allclose(
+        aet_ma_mean, nvals["aet"].values, rtol=1e-4, verbose=True
+    )
 
     tmpdir2 = TemporaryDirectory()
     gen_agg = AggGen(
         user_data=user_data,
-        stat_method="average",
+        stat_method="mean",
         agg_engine="parallel",
         agg_writer="csv",
         weights=tmpfile.name,
@@ -396,8 +430,9 @@ def test_odapcatdata_p(
     _ngdf, nvals2 = gen_agg.calculate_agg()
     file = Path(tmpdir2.name) / "test_agg_gen.csv"
     assert file.exists()
-    print(nvals2[0])
-    np.testing.assert_allclose(aet_average, nvals2[0], rtol=1e-4, verbose=True)
+    np.testing.assert_allclose(
+        aet_nan_mean, nvals2["aet"].values, rtol=1e-4, verbose=True
+    )
 
 
 def test_odapcatdata_serial_with_intersections(
@@ -406,8 +441,8 @@ def test_odapcatdata_serial_with_intersections(
     gdf: gpd.GeoDataFrame,
     poly_idx: str,
     wght_gen_proj: Any,
-    aet_ma_average: npt.NDArray[np.double],
-    aet_average: npt.NDArray[np.double],
+    aet_ma_mean: npt.NDArray[np.double],
+    aet_nan_mean: npt.NDArray[np.double],
 ) -> None:
     """Test ODAPCatData."""
     user_data = ODAPCatData(
@@ -432,7 +467,7 @@ def test_odapcatdata_serial_with_intersections(
 
     gen_agg = AggGen(
         user_data=user_data,
-        stat_method="masked_average",
+        stat_method="masked_mean",
         agg_engine="serial",
         agg_writer="netcdf",
         weights=tmpfile.name,
@@ -441,12 +476,14 @@ def test_odapcatdata_serial_with_intersections(
     )
 
     _ngdf, nvals = gen_agg.calculate_agg()
-    np.testing.assert_allclose(aet_ma_average, nvals[0], rtol=1e-4, verbose=True)
+    np.testing.assert_allclose(
+        aet_ma_mean, nvals["aet"].values, rtol=1e-4, verbose=True
+    )
 
     tmpdir2 = TemporaryDirectory()
     gen_agg2 = AggGen(
         user_data=user_data,
-        stat_method="masked_average",
+        stat_method="masked_mean",
         agg_engine="serial",
         agg_writer="csv",
         weights=tmpfile.name,
@@ -455,12 +492,14 @@ def test_odapcatdata_serial_with_intersections(
     )
 
     _ngdf2, nvals2 = gen_agg2.calculate_agg()
-    np.testing.assert_allclose(aet_ma_average, nvals2[0], rtol=1e-4, verbose=True)
+    np.testing.assert_allclose(
+        aet_ma_mean, nvals2["aet"].values, rtol=1e-4, verbose=True
+    )
 
     tmpdir3 = TemporaryDirectory()
     gen_agg3 = AggGen(
         user_data=user_data,
-        stat_method="average",
+        stat_method="mean",
         agg_engine="serial",
         agg_writer="csv",
         weights=tmpfile.name,
@@ -469,7 +508,9 @@ def test_odapcatdata_serial_with_intersections(
     )
 
     _ngdf3, nvals3 = gen_agg3.calculate_agg()
-    np.testing.assert_allclose(aet_average, nvals3[0], rtol=1e-4, verbose=True)
+    np.testing.assert_allclose(
+        aet_nan_mean, nvals3["aet"].values, rtol=1e-4, verbose=True
+    )
 
     file = Path(tmpdir.name) / "test_agg_gen.nc"
     assert file.exists()
@@ -485,8 +526,8 @@ def test_odapcatdata_s(
     gdf: gpd.GeoDataFrame,
     poly_idx: str,
     wght_gen_proj: Any,
-    aet_ma_average: npt.NDArray[np.double],
-    aet_average: npt.NDArray[np.double],
+    aet_ma_mean: npt.NDArray[np.double],
+    aet_nan_mean: npt.NDArray[np.double],
 ) -> None:
     """Test ODAPCatData."""
     user_data = ODAPCatData(
@@ -511,7 +552,7 @@ def test_odapcatdata_s(
 
     gen_agg = AggGen(
         user_data=user_data,
-        stat_method="masked_average",
+        stat_method="masked_mean",
         agg_engine="serial",
         agg_writer="netcdf",
         weights=tmpfile.name,
@@ -520,12 +561,14 @@ def test_odapcatdata_s(
     )
 
     _ngdf, nvals = gen_agg.calculate_agg()
-    np.testing.assert_allclose(aet_ma_average, nvals[0], rtol=1e-4, verbose=True)
+    np.testing.assert_allclose(
+        aet_ma_mean, nvals["aet"].values, rtol=1e-4, verbose=True
+    )
 
     tmpdir2 = TemporaryDirectory()
     gen_agg2 = AggGen(
         user_data=user_data,
-        stat_method="average",
+        stat_method="mean",
         agg_engine="serial",
         agg_writer="csv",
         weights=tmpfile.name,
@@ -534,12 +577,14 @@ def test_odapcatdata_s(
     )
 
     _ngdf2, nvals2 = gen_agg2.calculate_agg()
-    np.testing.assert_allclose(aet_average, nvals2[0], rtol=1e-4, verbose=True)
+    np.testing.assert_allclose(
+        aet_nan_mean, nvals2["aet"].values, rtol=1e-4, verbose=True
+    )
 
     tmpdir3 = TemporaryDirectory()
     gen_agg3 = AggGen(
         user_data=user_data,
-        stat_method="average",
+        stat_method="mean",
         agg_engine="serial",
         agg_writer="parquet",
         weights=tmpfile.name,
@@ -548,7 +593,9 @@ def test_odapcatdata_s(
     )
 
     _ngdf3, nvals3 = gen_agg3.calculate_agg()
-    np.testing.assert_allclose(aet_average, nvals3[0], rtol=1e-4, verbose=True)
+    np.testing.assert_allclose(
+        aet_nan_mean, nvals3["aet"].values, rtol=1e-4, verbose=True
+    )
 
     file = Path(tmpdir.name) / "test_agg_gen.nc"
     assert file.exists()
@@ -603,6 +650,51 @@ shp_poly_idx = "hru_id_nat"
 wght_gen_crs = 6931
 
 
+def test_weightgen_nofile(get_xarray: xr.Dataset, get_gdf: gpd.GeoDataFrame) -> None:
+    """Test WeightGen."""
+    user_data = UserCatData(
+        ds=get_xarray,
+        proj_ds=data_crs,
+        x_coord=x_coord,
+        y_coord=y_coord,
+        t_coord=t_coord,
+        var=tvar,
+        f_feature=get_gdf,
+        proj_feature=shp_crs,
+        id_feature=shp_poly_idx,
+        period=[sdate, edate],
+    )
+
+    wghtgen1 = WeightGen(
+        user_data=user_data,
+        method="serial",
+        output_file=None,
+        weight_gen_crs=wght_gen_crs,
+    )
+
+    wghts1 = wghtgen1.calculate_weights()
+
+    wghtgen2 = WeightGen(
+        user_data=user_data,
+        method="serial",
+        weight_gen_crs=wght_gen_crs,
+    )
+
+    wghts2 = wghtgen2.calculate_weights()
+
+    assert_frame_equal(wghts1, wghts2)
+
+    wghtgen3 = WeightGen(
+        user_data=user_data,
+        method="serial",
+        weight_gen_crs=wght_gen_crs,
+        output_file="",
+    )
+
+    wghts3 = wghtgen3.calculate_weights()
+    assert_frame_equal(wghts1, wghts3)
+
+
 def test_usercatdata(
     get_xarray: xr.Dataset,
     get_gdf: gpd.GeoDataFrame,
@@ -660,17 +752,16 @@ def test_usercatdata(
 
     agg_gen = AggGen(
         user_data=user_data2,
-        stat_method="average",
+        stat_method="mean",
         agg_engine="serial",
         agg_writer="csv",
-        weights=wghts,
+        weights=_wghts2,
         out_path=tmpdir.name,
         file_prefix="test_agg_gen_2",
         jobs=2,
     )
 
     _ngdf, _nvals = agg_gen.calculate_agg()
-    print(_nvals[0])
 
     user_data3 = UserCatData(
         ds=get_xarray,
@@ -687,12 +778,12 @@ def test_usercatdata(
     tmpdir2 = TemporaryDirectory()
     agg_gen = AggGen(
         user_data=user_data3,
-        stat_method="masked_average",
+        stat_method="masked_mean",
         agg_engine="serial",
         agg_writer="netcdf",
         weights=wghts,
         out_path=tmpdir2.name,
-        file_prefix="test_agg_gen_2",
+        file_prefix="test_agg_gen_3",
         jobs=2,
     )
 
@@ -701,3 +792,75 @@ def test_usercatdata(
     outfile = Path(tmpdir.name) / "test_agg_gen_2.csv"
 
     assert outfile.exists()
+
+
+def test_interp_gen(param_dict: dict[str, Any], grid_dict: dict[str, Any]) -> None:
+    """Test InterpGen."""
+    lines = gpd.read_file("./tests/data/test_lines.json")
+    user_data = ODAPCatData(
+        param_dict=param_dict,
+        grid_dict=grid_dict,
+        f_feature=lines,
+        id_feature="Permanent_Identifier",
+        period=["1980-10-01", "1980-10-01"],
+    )
+
+    interp_object = InterpGen(
+        user_data, pt_spacing=0, stat="all", return_sample_geom=True, calc_crs=5070
+    )
+
+    stats, pts = interp_object.calc_interp()
+    assert isinstance(stats, pd.DataFrame)
+    assert isinstance(pts, gpd.GeoDataFrame)
+
+    # assert stats[0]["mean"] == pytest.approx(69.09591, 0.001)
+    # assert stats[0]["median"] == pytest.approx(69.3, 0.001)
+    # assert stats[0]["std"] == pytest.approx(0.47364053, 0.001)
+    # assert stats[0]["min"] == pytest.approx(68.4, 0.001)
+    # assert stats[0]["max"] == pytest.approx(69.6, 0.001)
+
+    # assert stats[1]["mean"] == pytest.approx(54, 0.001)
+    # assert stats[1]["median"] == pytest.approx(54, 0.001)
+    # assert stats[1]["std"] == pytest.approx(0, 0.001)
+    # assert stats[1]["min"] == pytest.approx(54, 0.001)
+    # assert stats[1]["max"] == pytest.approx(54, 0.001)
+
+    # assert stats[2]["mean"] == pytest.approx(83.59093, 0.001)
+    # assert stats[2]["median"] == pytest.approx(83.6, 0.001)
+    # assert stats[2]["std"] == pytest.approx(0.17342508, 0.001)
+    # assert stats[2]["min"] == pytest.approx(83.4, 0.001)
+    # assert stats[2]["max"] == pytest.approx(84.1, 0.001)
+
+    # assert stats[3]["mean"] == pytest.approx(82.20001, 0.001)
+    # assert stats[3]["median"] == pytest.approx(82.200005, 0.001)
+    # assert stats[3]["std"] == pytest.approx(9.918213e-05, 0.001)
+    # assert stats[3]["min"] == pytest.approx(82.200005, 0.001)
+    # assert stats[3]["max"] == pytest.approx(82.200005, 0.001)
+
+    # assert stats[4]["mean"] == pytest.approx(-0.8652154, 0.001)
+    # assert stats[4]["median"] == pytest.approx(-0.85999995, 0.001)
+    # assert stats[4]["std"] == pytest.approx(0.02629874, 0.001)
+    # assert stats[4]["min"] == pytest.approx(-0.91999996, 0.001)
+    # assert stats[4]["max"] == pytest.approx(-0.81, 0.001)
+
+    # assert stats[5]["mean"] == pytest.approx(-2.6699998, 0.001)
+    # assert stats[5]["median"] == pytest.approx(-2.6699998, 0.001)
+    # assert stats[5]["std"] == pytest.approx(2.861023e-06, 0.001)
+    # assert stats[5]["min"] == pytest.approx(-2.6699998, 0.001)
+    # assert stats[5]["max"] == pytest.approx(-2.6699998, 0.001)
+
+    # assert len(pts[0].keys()) == 441
+    # # assert pts[0][1] == pytest.approx(68.400002, 0.001)
+    # assert pts[0][1].distance == pytest.approx(317.080061, 0.001)
+    # # assert pts[2][3] == pytest.approx(83.400002, 0.001)
+    # assert pts[2][3].distance == pytest.approx(443.342027, 0.001)
+    # # assert pts[4][5].PDSI == pytest.approx(-0.92, 0.001)
+    # assert pts[4][5].distance == pytest.approx(465.363614, 0.001)
+
+    # assert len(pts[1].keys()) == 133
+    # # assert pts[1][1].aet == pytest.approx(54.0, 0.001)
+    # assert pts[1][1].distance == pytest.approx(7.628603, 0.001)
+    # # assert pts[3][3].pet == pytest.approx(82.200005, 0.001)
+    # assert pts[3][3].distance == pytest.approx(20.146425, 0.001)
+    # # assert pts[5][5].PDSI == pytest.approx(-2.67, 0.001)
+    # assert pts[5][5].distance == pytest.approx(43.541266, 0.001)
